# Provides a convenient way to store and retrieve permission data
class Permify::PermissionMap
  def initialize
    @data = {}
  end

  def store(resource, action, info)
    resource_key, action_key = to_keys(resource, action)
    data[resource_key] ||= {}
    data[resource_key][action_key] = info
  end

  def find(resource, action)
    resource_key, action_key = to_keys(resource, action)
    data.fetch(resource_key, {})[action_key]
  end

  private
  attr_reader :data

  # Transforms resource and action into keys for use in internal hashes
  #
  # @param resource [Class, Object] resource object or its class
  # @param action [String, #to_sym]
  #
  # @return keys [Array<Symbol>] array with resource and action key
  #
  def to_keys(resource, action)
    resource_class = resource.is_a?(Class) ? resource : resource.class
    [resource_class.name, action].map(&:to_sym)
  end
end
