class Permify::Repository

  # Define a static permission
  #
  # @param resource_class [Class]
  # @param action [Symbol, String]
  #
  # @return [undefined]
  #
  def static(resource_class, action)
    permission = Permify::Permission::Static.new
    permissions.store(resource_class, action, permission)
  end

  # Define a dynamic permission
  #
  # @param resource_class [Class]
  # @param action [Symbol, String]
  #
  # @return [undefined]
  #
  def dynamic(resource_class, action, *args)
    permission = Permify::Permission::Dynamic.new args.flatten
    permissions.store(resource_class, action, permission)
  end

  # Creates a permission that's granted whenever any of the combined
  # permissions is granted
  #
  # @param resource_class [Class]
  # @param action [String, #to_sym]
  #
  # @return [undefined]
  #
  def any(resource_class, action, combinations)
    combination_map = Permify::Permission::CombinationMap.new(combinations)
    permission_combinations.store(resource_class, action, combination_map)
  end

  # Finds permission in this repository
  #
  # @param resource [Class]
  # @param action [String, #to_sym]
  #
  # @return permission [Permify::Permission, nil]
  #
  def find_permission(resource, action)
    permissions.find(resource, action)
  end

  def permission_combination?(resource_class, action)
    !!find_permission_combination(resource_class, action)
  end

  def find_permission_combination(resource, action)
    permission_combinations.find(resource, action)
  end

  protected

  def permissions
    @permissions ||= Permify::PermissionMap.new
  end

  def permission_combinations
    @permission_combinations ||= Permify::PermissionMap.new
  end
end
